# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

import sys
from pathlib import Path

from cleez.command import Command

STD_FILES = [
    ".git/",
    ".gitignore",
    ".pre-commit-config.yaml",
    "Makefile",
    "README.md",
    # noxfile.py (or tox.ini)
    "poetry.lock",
    "pyproject.toml",
    "ruff.toml",
    "src/",
    "setup.cfg",
    "tests/",
]


class CruftCommand(Command):
    """Run cruft audit."""

    name = "cruft"

    def run(self):
        self.check_standard_files()

    def check_standard_files(self):
        """Check standard files ("cruft") are present."""
        success = True

        for std_file in STD_FILES:
            if not Path(std_file).exists():
                print(f"Missing standard file: {std_file}")
                success = False

        if not Path("tox.init").exists() and not Path("noxfile.py").exists():
            print("Missing tox.ini or noxfile.py")
            success = False

        if not success:
            sys.exit(1)



"""Experimental: reusable invoke tasks."""

import re
from pathlib import Path

from cleez import Command


class HelpMake(Command):
    """Helper to generate the `make help` message."""

    name = "help-make"

    def run(self):
        help_make()


# Still used by invoke help-make (deprecated)
def help_make():
    with Path("Makefile").open() as f:
        makefile = f.read()

    targets = MakefileParser().parse(makefile)

    if not targets:
        print("No documented targets found in Makefile")
        return

    max_len = max(len(t[0]) for t in targets)

    print("Documented targets:\n")
    for target, description in targets:
        print(f"  {target:<{max_len}}   {description}")


class MakefileParser:
    def __init__(self):
        self.target = ""
        self.description = ""
        self.targets = []

    def parse(self, makefile: str):
        for line in makefile.splitlines():
            self.parse_line(line)
        return self.targets

    def parse_line(self, line: str):
        if line.startswith(".PHONY:"):
            return

        if m := re.match(r"^## (.*)", line):
            self.description = m.group(1)
        elif m := re.match(r"^(\S*?):", line):
            self.target = m.group(1)
            if self.description:
                self.targets.append([self.target, self.description])
        else:
            self.target = ""
            self.description = ""



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT
import shutil
from pathlib import Path

from cleez.colors import bold, dim
from cleez.command import Command

CRUFT_DIRS = [".mypy_cache", ".pytest_cache", ".ruff_cache"]


class CleanCommand(Command):
    """Cleanup cruft."""

    name = "clean"

    def run(self):
        print(bold("Removing Python bytecode cache directories..."))
        for cache_dir in Path().rglob("**/__pycache__"):
            shutil.rmtree(cache_dir)

        print(bold("Removing other caches..."))
        for cache_dir in CRUFT_DIRS:
            if Path(cache_dir).exists():
                print(dim(f"Removing {cache_dir}"))
                shutil.rmtree(cache_dir, ignore_errors=True)



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT
from pathlib import Path

from cleez.colors import red


def check_files_exist(args):
    for arg in args:
        if not Path(arg).exists():
            print(red(f"{arg} does not exist"))
            raise SystemExit(1)



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

from cleez.command import Command

from ..shell import run


class AuditCommand(Command):
    """Run security audit."""

    name = "audit"

    def run(self):
        run("pip-audit")
        # TODO: don't assume source dir is src
        run("bandit -q -c pyproject.toml -r src")
        # TODO: suppress output on success
        run("reuse lint")
        # TODO: Don't run safety check for now, it's too noisy
        # run("safety check")



"""Experimental: reusable invoke tasks."""

from collections.abc import Sequence
from typing import Any, Optional

from invoke import task

from . import help, versions

MODULES = [help, versions]


def import_tasks(namespace: dict[str, Any], modules: Optional[Sequence[str]] = None):
    """Import tasks from other modules."""
    if modules is None:
        _modules = MODULES
    else:
        _modules = [globals()[module_name] for module_name in modules]
    for module in _modules:
        import_tasks_from_module(namespace, module)


def import_tasks_from_module(namespace, module):
    for task_name in module.TASKS:
        _task = getattr(module, task_name)
        namespace[_task.__name__] = task(_task)



import sys
from pathlib import Path
from time import gmtime, strftime

import tomlkit
from cleez import Argument, Command

from ..shell import run


class BumpVersionCommand(Command):
    """Bump version in pyproject.toml, commit & apply tag.

    Parameter "rule" can be one of: "daily" or a parameter accepted by
    "poetry version".
    """

    name = "bump-version"
    help = "Bump version in pyproject.toml, commit & apply tag."

    arguments = [
        Argument(
            "rule",
            nargs="?",
            default="patch",
            help="Bump rule (patch, minor, major, daily)",
        ),
    ]

    def run(self, rule: str = "patch"):
        return_code = run("git diff --quiet", warn=True)
        if return_code != 0:
            print("Your repo is dirty. Please commit or stash changes first.")
            sys.exit(1)

        update_version(rule)
        version = get_version()
        run("git add pyproject.toml")
        run(f"git commit -m 'Bump version ({version})'")
        run(f"git tag {version}")


def update_version(rule):
    if rule == "daily":
        update_version_daily()
    else:
        run(f"poetry version {rule}")


def update_version_daily():
    # NB: we're using tomkit for its roundtrip feature.
    pyproject_d = tomlkit.parse(Path("pyproject.toml").read_text())
    version: str = pyproject_d["tool"]["poetry"]["version"]  # type: ignore
    serial = int(version.split(".")[-1])
    date = version[: -len(str(serial)) - 1]

    new_date = strftime("%Y.%m.%d", gmtime())
    if date == new_date:
        serial += 1
    else:
        serial = 1

    new_version = f"{new_date}.{serial}"
    print("New version: ", new_version)
    pyproject_d["tool"]["poetry"]["version"] = new_version  # type: ignore
    Path("pyproject.toml").write_text(tomlkit.dumps(pyproject_d))


def get_version():
    d = tomlkit.parse(Path("pyproject.toml").read_text())
    version: str = d["tool"]["poetry"]["version"]  # type: ignore
    return version
